from dataclasses import dataclass
from typing import Optional

from ttss.Trip import Trip


@dataclass
class Vehicle:
    id: Optional[str] = None
    active: Optional[bool] = None
    latitude: Optional[float] = None
    longitude: Optional[float] = None
    heading: Optional[int] = None
    category: Optional[str] = None
    color: Optional[str] = None
    trip: Optional[Trip] = None



from datetime import time, datetime
from time import time_ns


def parse_time(string: str) -> time:
    return datetime.strptime(string, '%H:%M').time()


def timestamp_ms() -> int:
    return time_ns() // 1_000_000



from dataclasses import dataclass
from typing import Optional


@dataclass
class StopPoint:
    id: Optional[str] = None
    name: Optional[str] = None
    code: Optional[str] = None
    category: Optional[str] = None
    latitude: Optional[float] = None
    longitude: Optional[float] = None



from dataclasses import dataclass
from typing import Optional

from ttss.Route import Route


@dataclass
class Trip:
    id: Optional[str] = None
    route: Optional[Route] = None
    direction: Optional[str] = None



from dataclasses import dataclass
from datetime import datetime, time
from typing import Optional

from ttss.Route import Route
from ttss.Status import Status
from ttss.Stop import Stop
from ttss.Trip import Trip
from ttss.Vehicle import Vehicle


@dataclass
class Passage:
    id: Optional[str] = None
    old: Optional[bool] = None
    status: Optional[Status] = None
    planned_time: Optional[time] = None
    actual_time: Optional[time] = None
    dt: Optional[datetime] = None
    seq_num: Optional[int] = None
    stop: Optional[Stop] = None
    trip: Optional[Trip] = None
    route: Optional[Route] = None
    vehicle: Optional[Vehicle] = None



import re
from datetime import timedelta, datetime
from html import unescape
from typing import List, Dict, Tuple, Optional, Any

from ttss.Passage import Passage
from ttss.Path import Path
from ttss.Route import Route
from ttss.Status import Status
from ttss.Stop import Stop
from ttss.StopPoint import StopPoint
from ttss.Trip import Trip
from ttss.Vehicle import Vehicle
from ttss.utils import parse_time


def extract_autocomplete_stops(html_text: str, /) -> List[Stop]:
    pattern = re.compile(r'<li stop="(\d+)">(.*)</li>')
    matches = re.finditer(pattern, html_text)
    return [
        Stop(number=match.group(1), name=unescape(match.group(2)))
        for match in matches
    ]


def extract_autocomplete_stops_json(data: dict, /) -> List[Stop]:
    return [
        Stop(number=stop['id'], name=unescape(stop['name']))
        for stop in data[1:]
    ]


def extract_stops(data: dict, /) -> List[Stop]:
    return [
        Stop(id=stop['id'],
             name=stop['name'],
             number=stop['shortName'],
             category=stop['category'],
             latitude=stop['latitude'] / 3_600_000,
             longitude=stop['longitude'] / 3_600_000)
        for stop in data['stops']
    ]


def extract_stop_points(data: dict, /) -> List[StopPoint]:
    return [
        StopPoint(id=stop_point['id'],
                  name=stop_point['name'],
                  code=stop_point['stopPoint'],
                  category=stop_point['category'],
                  latitude=stop_point['latitude'] / 3_600_000,
                  longitude=stop_point['longitude'] / 3_600_000)
        for stop_point in data['stopPoints']
    ]


def extract_stop(data: dict, /) -> Stop:
    return Stop(id=data['id'], name=data['passengerName'])


def extract_stop_point(data: dict, /) -> StopPoint:
    return StopPoint(id=data['id'], name=data['passengerName'], code=data['stopPointCode'])


def extract_stop_passages(data: dict, /, *, now: datetime) -> Tuple[Stop, List[Passage]]:
    stop = Stop(name=data['stopName'])
    passages = extract_stop_passages_list(data['old'], stop=stop, now=now, old=True) + \
               extract_stop_passages_list(data['actual'], stop=stop, now=now, old=False)  # noqa
    return stop, passages


def extract_stop_passages_list(passages: List[Dict[str, Any]], /, *,
                               stop: Stop, now: datetime, old: bool) -> List[Passage]:
    return [extract_stop_passage(passage, stop=stop, now=now, old=old) for passage in passages]


def extract_stop_passage(passage: Dict[str, Any], /, *, stop: Stop, now: datetime, old: bool) -> Passage:
    route = Route(id=passage['routeId'], name=passage['patternText'])

    trip = Trip(id=passage['tripId'],
                route=route,
                direction=passage['direction'])

    vehicle = Vehicle(id=passage['vehicleId'], trip=trip) if 'vehicleId' in passage else None

    return Passage(id=passage['passageid'],
                   stop=stop,
                   trip=trip,
                   route=route,
                   vehicle=vehicle,
                   planned_time=parse_time(passage['plannedTime']),
                   actual_time=parse_time(passage['actualTime']) if 'actualTime' in passage else None,
                   dt=now + timedelta(seconds=passage['actualRelativeTime']),
                   status=Status(passage['status']),
                   old=old)


def extract_stop_point_passages(data: dict, /, *, now: datetime) -> Tuple[Stop, List[Passage]]:
    return extract_stop_passages(data, now=now)


def extract_trip_passages_list(passages: List[Dict[str, Any]], /, *, trip: Trip, old: bool) -> List[Passage]:
    return [extract_trip_passage(passage, trip=trip, old=old) for passage in passages]


def extract_trip_passage(data: dict, /, *, trip: Trip, old: bool) -> Passage:
    stop = Stop(id=data['stop']['id'],
                name=data['stop']['name'],
                number=data['stop']['shortName'])

    return Passage(stop=stop,
                   seq_num=int(data['stop_seq_num']),
                   actual_time=parse_time(data['actualTime']),
                   status=Status(data['status']),
                   trip=trip,
                   route=trip.route,
                   old=old)


def extract_trip_passages(data: dict, /) -> Tuple[Optional[Trip], List[Passage]]:
    route = Route(name=data.get('routeName', None))
    trip = Trip(route=route, direction=data.get('directionText', None))
    passages = extract_trip_passages_list(data['old'], trip=trip, old=True) + \
               extract_trip_passages_list(data['actual'], trip=trip, old=False)  # noqa
    return trip, passages


def extract_routes(data: dict, /) -> List[Route]:
    return [extract_route(route) for route in data['routes']]


def extract_route_stops(data: dict, /) -> Tuple[Route, List[Stop]]:
    route = extract_route(data['route'])
    stops = [
        Stop(id=stop['id'], name=stop['name'], number=stop['number'])
        for stop in data['stops']
    ]
    return route, stops


def extract_route(data: dict, /) -> Route:
    return Route(id=data['id'],
                 name=data['name'],
                 authority=data['authority'],
                 directions=data.get('directions', []),
                 alerts=data['alerts'])


def extract_route_paths(data: dict, /) -> List[Path]:
    return extract_paths(data)


def extract_vehicle_paths(data: dict, /) -> List[Path]:
    return extract_paths(data)


def extract_paths(data: dict, /) -> List[Path]:
    return [extract_path(path) for path in data['paths']]


def extract_path(data: dict, /) -> Path:
    waypoints = [
        (point['lat'] / 3_600_000, point['lon'] / 3_600_000)
        for point in data['wayPoints']
    ]
    return Path(color=data['color'], waypoints=waypoints)


def extract_vehicles(data: dict, /) -> List[Vehicle]:
    return [extract_vehicle(vehicle) for vehicle in data['vehicles']]


def extract_vehicle(data: dict, /) -> Vehicle:
    active = 'isDeleted' not in data

    latitude = data['latitude'] / 3_600_000 if 'latitude' in data else None
    longitude = data['longitude'] / 3_600_000 if 'longitude' in data else None

    if 'name' in data:
        route_number, direction = data['name'].split(' ', 1)
    else:
        route_number = direction = None

    if 'tripId' in data:
        route = Route(name=route_number)
        trip = Trip(id=data.get('tripId', None),
                    route=route,
                    direction=direction)
    else:
        trip = None

    return Vehicle(id=data['id'],
                   active=active,
                   category=data.get('category', None),
                   latitude=latitude,
                   longitude=longitude,
                   heading=data.get('heading', None),
                   color=data.get('color', None),
                   trip=trip)



from dataclasses import dataclass
from datetime import datetime
from typing import Tuple, List, Optional

import requests

from ttss.ColorType import ColorType
from ttss.Mode import Mode
from ttss.Passage import Passage
from ttss.Path import Path
from ttss.PositionType import PositionType
from ttss.Route import Route
from ttss.Stop import Stop
from ttss.StopPoint import StopPoint
from ttss.Trip import Trip
from ttss.Vehicle import Vehicle
from ttss.extractors import extract_autocomplete_stops, extract_autocomplete_stops_json, extract_stops, \
    extract_stop_points, extract_stop, extract_stop_point, extract_stop_passages, extract_stop_point_passages, \
    extract_trip_passages, extract_routes, extract_route_stops, extract_route_paths, extract_vehicle_paths, \
    extract_vehicles
from ttss.utils import timestamp_ms


@dataclass
class TTSS:
    base_url: str

    def autocomplete_stops(self, query: str) -> List[Stop]:
        url = f'{self.base_url}/internetservice/services/lookup/autocomplete'
        params = {
            'query': query,
            'language': 'pl',
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_autocomplete_stops(response.text)

    def autocomplete_stops_json(self, query: str) -> List[Stop]:
        url = f'{self.base_url}/internetservice/services/lookup/autocomplete/json'
        params = {
            'query': query,
            'language': 'pl',
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_autocomplete_stops_json(response.json())

    def get_stops(self, *,
                  min_latitude: float = -90.0, max_latitude: float = 90.0,
                  min_longitude: float = -180.0, max_longitude: float = 180.0) -> List[Stop]:
        url = f'{self.base_url}/internetservice/geoserviceDispatcher/services/stopinfo/stops'
        params = {
            'left': int(min_longitude * 3_600_000),
            'bottom': int(min_latitude * 3_600_000),
            'right': int(max_longitude * 3_600_000),
            'top': int(max_latitude * 3_600_000),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_stops(response.json())

    def get_stop_points(self, *,
                        min_latitude: float = -90.0, max_latitude: float = 90.0,
                        min_longitude: float = -180.0, max_longitude: float = 180.0) -> List[StopPoint]:
        url = f'{self.base_url}/internetservice/geoserviceDispatcher/services/stopinfo/stopPoints'
        params = {
            'left': int(min_longitude * 3_600_000),
            'bottom': int(min_latitude * 3_600_000),
            'right': int(max_longitude * 3_600_000),
            'top': int(max_latitude * 3_600_000),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_stop_points(response.json())

    def get_stop(self, stop_number: str) -> Optional[Stop]:
        url = f'{self.base_url}/internetservice/services/stopInfo/stop'
        params = {
            'stop': stop_number,
            'language': 'pl',
        }
        response = requests.post(url, params)
        if response.status_code == 404:
            return None
        response.raise_for_status()
        return extract_stop(response.json())

    def get_stop_point(self, stop_point_code: str) -> Optional[StopPoint]:
        url = f'{self.base_url}/internetservice/services/stopInfo/stopPoint'
        params = {
            'stopPoint': stop_point_code,
            'language': 'pl',
        }
        response = requests.post(url, params)
        if response.status_code == 404:
            return None
        response.raise_for_status()
        return extract_stop_point(response.json())

    def get_stop_passages(self, stop_number: str, *,
                          mode: Mode = Mode.DEPARTURES, timeframe: int = 120) -> Tuple[Stop, List[Passage]]:
        now = datetime.now().replace(microsecond=0)
        url = f'{self.base_url}/internetservice/services/passageInfo/stopPassages/stop'
        params = {
            'language': 'pl',
            'stop': stop_number,
            'mode': mode.value,
            'timeFrame': timeframe,
            'cacheBuster': timestamp_ms(),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_stop_passages(response.json(), now=now)

    def get_stop_point_passages(self, stop_point_id: str, *,
                                mode: Mode = Mode.DEPARTURES, timeframe: int = 120) -> Tuple[Stop, List[Passage]]:
        now = datetime.now().replace(microsecond=0)
        url = f'{self.base_url}/internetservice/services/passageInfo/stopPassages/stopPoint'
        params = {
            'language': 'pl',
            'stopPoint': stop_point_id,
            'mode': mode.value,
            'timeFrame': timeframe,
            'cacheBuster': timestamp_ms(),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_stop_point_passages(response.json(), now=now)

    def get_trip_passages(self, trip_id: str, *, vehicle_id: Optional[str] = None,
                          mode: Mode = Mode.DEPARTURES) -> Tuple[Optional[Trip], List[Passage]]:
        url = f'{self.base_url}/internetservice/services/tripInfo/tripPassages'
        params = {
            'language': 'pl',
            'tripId': trip_id,
            'mode': mode.value,
            'vehicleId': vehicle_id,
            'cacheBuster': timestamp_ms(),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_trip_passages(response.json())

    def get_routes(self) -> List[Route]:
        url = f'{self.base_url}/internetservice/services/routeInfo/route'
        params = {'language': 'pl'}
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_routes(response.json())

    def get_route_stops(self, route_id: str) -> Tuple[Route, List[Stop]]:
        url = f'{self.base_url}/internetservice/services/routeInfo/routeStops'
        params = {
            'routeId': route_id,
            'language': 'pl',
            'cacheBuster': timestamp_ms(),
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_route_stops(response.json())

    def get_route_paths(self, route_id: str, *, direction: Optional[str] = None) -> List[Path]:
        url = f'{self.base_url}/internetservice/geoserviceDispatcher/services/pathinfo/route'
        params = {
            'id': route_id,
            'direction': direction,
        }
        response = requests.post(url, params)
        if response.status_code == 404:
            return []
        response.raise_for_status()
        return extract_route_paths(response.json())

    def get_vehicle_paths(self, vehicle_id: str) -> List[Path]:
        url = f'{self.base_url}/internetservice/geoserviceDispatcher/services/pathinfo/vehicle'
        params = {'id': vehicle_id}
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_vehicle_paths(response.json())

    def get_vehicles(self, *,
                     last_update: Optional[int] = None,
                     position_type: PositionType = PositionType.CORRECTED,
                     color_type: ColorType = ColorType.ROUTE_BASED) -> List[Vehicle]:
        url = f'{self.base_url}/internetservice/geoserviceDispatcher/services/vehicleinfo/vehicles'
        params = {
            'lastUpdate': last_update,
            'positionType': position_type,
            'colorType': color_type,
        }
        response = requests.post(url, params)
        response.raise_for_status()
        return extract_vehicles(response.json())

    def get_language(self) -> None:
        # url = f'{self.base_url}/internetservice/messages/language'
        raise NotImplementedError()



import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='ttss',
    version='0.2.0',
    author='Tomek Zawadzki',
    author_email='tomekzawadzki98@gmail.com',
    description='A Python wrapper for TTSS API.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tomekzaw/ttss',
    project_urls={
        'Bug Tracker': 'https://github.com/tomekzaw/ttss/issues',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    python_requires='>=3.8',
)



from dataclasses import dataclass
from typing import List, Optional


@dataclass
class Route:
    id: Optional[str] = None
    name: Optional[str] = None
    authority: Optional[str] = None
    directions: Optional[List[str]] = None
    alerts: Optional[List[str]] = None
